/*
    bsleep (Blacky Sleep)

    Dieses Programm wartet eine anzahl von Secunden und zeigt dabei
    einen Contdown an.

    Copyright (C) Andreas Hofmeier

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation; either version 2 of
    the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


    Name:     Andreas Hofmeier

    Adresse:  Stralsunder Strae 20
              28239 Bremen
              Germany

    Telefon:  +49-421-613793

    eMail:    An-H@An-H.de
              hofmeier@inbremen.de
              an-h@gmx.de

    Homepage: http://www.An-H.de/
	      hofmeier.home.pages.de
              http://www.LGut.uni-bremen.de/anh/


*/


#include <stdio.h>
#include <time.h>
#include <math.h>
#include "hofmeier.h"
#include <curses.h>
#include <signal.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>


char blacky_ver_info[] = "Blacky's Sleep v0.04 (c) Copyright 1998-2000,2003 Andreas Hofmeier";

static void finish(int sig);


main(argc, argv)
  int argc;
  char *argv[];
{
  int i;
  long waittime = 0;
  long waittimec = 0;
  char *msg = NULL;
  char flashing = 1; //flashing while display msg?
  char *prg = NULL;
  char bell = 0;
  char countdown = 1;


  if (argc < 2 || ((argc > 1) ? ((argv[1][0] == '-') && ((argv[1][1] == '?') || (toupper(argv[1][1]) == 'H'))) : 0)) {
    allblackyintro(blacky_ver_info);
    fprintf(stderr, "Help:\n");
    fprintf(stderr, "  bsleep <-v | -h | <n>[smhd]...> [options]\n");
    fprintf(stderr, "    -v        : Zeigt Licenze und Version\n");
    fprintf(stderr, "    <n>       : wait n seconds.\n");
    fprintf(stderr, "    <n>s      : wait n second.\n");
    fprintf(stderr, "    <n>m      : wain n minutes.\n");
    fprintf(stderr, "    <n>h      : wait n hours.\n");
    fprintf(stderr, "    <n>d      : wait n days.\n");
    fprintf(stderr, "    -b        : bell one times after wain.\n");
    fprintf(stderr, "    -m <msg>  : display message msg and wait belling and flashing\n                for confirmation after wait\n");
    fprintf(stderr, "    -f        : don't flashing while showing the message.\n");
    fprintf(stderr, "    -e <p>    : launch program <p> after waiting.\n");
    fprintf(stderr, "    -q        : don't show a countdown.\n");
    exit(1);
  }

  if ((argv[1][0] == '-') && (toupper(argv[1][1]) == 'V')) {
    printf("%s\n\n", blacky_ver_info);
    printkinfo(stdout);
    exit(0);
  }

  for (i = 0; i < argc; i++) {
    //Eine Zeitspanne gefunden...
    if (isdigit(argv[i][0])) {
      int mul = 0;
      int len = strlen(argv[i]);
      switch (argv[i][len - 1]) {
      case 's':
      case 'S':
	mul = 1;
	break;
      case 'm':
      case 'M':
        mul = 60;
        break;
      case 'h':
      case 'H':
        mul = 60 * 60;
        break;
      case 'd':
      case 'D':
        mul = 60 * 60 * 24;
        break;
      } // sitch()
      if (mul > 0) {
	argv[i][len - 1] = 0;
      } else {
	mul = 1;
      }
      
      waittime += atol(argv[i]) * mul;
    }

    if (argv[i][0] == '-') {
      switch (argv[i][1]) {
      case 'm':
	if (argv[i][2] != 0) {
	  msg = argv[i] + 2;
	} else {
	  msg = argv[++i];
	}
	break;
      case 'f':
	flashing = 0;
	break;
      case 'b':
	bell = 1;
	break;
      case 'e':
	if (argv[i][2] != 0) {
	  prg = argv[i] + 2;
	} else {
	  prg = argv[++i];
	}
	break;
      case 'q':
	countdown = 0;
	break;
      } // switch()
    }
  }

  if (countdown) {
    /* Runterzhlen */
    waittimec = waittime;
    for (  ; waittimec > 0; waittimec--) {
    printf("%cWaiting: %s ", 13, printz(waittimec));
    printf("of %s secounds...", printz(waittime));
    //      printf("%s%c%c%c%c%c%c%c%c", printz(waittime), 8, 8, 8, 8, 8, 8, 8, 8);
      fflush(stdout);
      sleep(1);
    }
    printf("\n");
  } else {
    waittimec = waittime;
    for (  ; waittimec > 0; waittimec--) {
      if (waittimec >= 65535) {
	sleep(65535);
	waittimec -= 65535;
      } else {
	sleep(waittimec);
      }
    }
  }

  if (bell) {
    /* Beep */
    fprintf(stderr, "%c", 7);
    fflush(stderr);
  }

  if (msg != NULL) {
    // select()
    fd_set rfds;
    struct timeval tv;
    int retval;

    int invert = 0;
    int msglen = strlen(msg);

    if (flashing) {
      init_cur_graph();
    }
    signal(SIGINT, finish);      /* arrange interrupts to terminate */
    signal(SIGKILL, finish);
    signal(SIGTERM, finish);

    if (!flashing) {
      puts("Press any key to continue!");
      if (msg != NULL) {
	puts(msg);
      }
      system("/bin/stty cbreak");
    }
    while (1) {
      if (flashing) {
	static int i, i2;

	if (invert == 0) {
	  attrset(A_STANDOUT);
	  invert = 1;
	} else {
	  attrset(A_NORMAL);
	  invert = 0;
	}
	for (i = 0; i < LINES; i++) {
	  gotoxy(1, i + 1);
	  for (i2 = COLS; i2 > 0; i2--) {
	    addstr(" ");
	  }
	}
	gotoxy(COLS/2 - 13, LINES/2);
	addstr("Press any key to continue!");
	if (msg != NULL) {
	  gotoxy(COLS/2 - msglen/2, LINES/2+1);
	  addstr(msg);
	}
	refresh();
      }
      putchar(7);
      fflush(stdout);
      
      FD_ZERO(&rfds);
      FD_SET(0, &rfds);
      tv.tv_sec = 1;
      tv.tv_usec = 0;
      retval = select(1, &rfds, NULL, NULL, &tv);
      if (retval) {
	break;
      }
    }
    finish(0);
  }

  if (prg != NULL) {
    system(prg);
  }
  
  exit(0);
}


static void finish(int sig)
{
  endwin();
  system("/bin/stty -cbreak");
  exit(0);
}
